#ifdef X11
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#else
#include <vga.h>
#endif

#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include "tetX.h"


#include "kbcodes.h"
#define Esc 0x1B


/*===================================================*/
/*          Joystick and keyboard processing         */
/*---------------------------------------------------*/
/*       04.04.98                          M.G.      */
/*===================================================*/
#ifdef JOYSTICK_SUPPORTED
static int JoyX1=0, JoyX2=0, JoyY1=0, JoyY2=0;
static int JoyX=0, JoyY=0, Buttons=0;
static int JoyButtonLocked;
char JoyName[32] = "unknown";
/*
#warning  "If joystick support is not included in your"
#warning  "system, you need to comment out line "
#warning  "'#define JOYSTICK_SUPPORTED' in tetconfig.h,"
#warning  "otherwise this file will compile with errors"
*/
#include <linux/joystick.h>
#include <sys/ioctl.h>
#ifdef JS_VERSION  
#if (JS_VERSION >= 0x010208) && (JOYSTICK_SUPPORTED == 1)
#define NEW_JOYSTICK 
#define NEW_JOY_MAX 32767l
#endif   /* JS_VERSION > 0x010208 */
#endif   /* JS_VERSION */
#else   /* JOYSTICK_SUPPORTED */
char JoyName[] = "not supported";
#endif

extern int  JoyEnabled;                // tetgame.c
static int JoyFd = -1;
int  JoyVersion = 0;
short  UseNewDriver = 1;
short  JoyNewDriver = 0;

/* Get a key from the keyboard if available
   For special keys returns
        LSB:  ASCI code or last byte in ANSI sequence
        MSB:  0 - ASCII character
	        1 - arrow key
              2 - Alt key
	        3 - Functional  key (F1, F2, etc) with no CTRL
*/

#ifdef X11
extern  Display		*display;
extern  Window		window;

unsigned int  kbdGet(void)
{
      XEvent event; 
      char str[3]; 
      int  n;
      KeySym keysym;

      if (!XPending(display)) return 0;


      /* First we have to look for all non kbd events */
      if (ProceedSystemEvents()) return 0;
      ProceedExposures();
      if (GetWindowState() == IconicState) SuspendGame();

      /* Now, check if we have a keyboard event */
      if (! XCheckTypedEvent(display, KeyPress, &event) ||
            XFilterEvent(&event, window)) return 0;  
      
      n = XLookupString(&(event.xkey), str, 2, &keysym, (XComposeStatus *)NULL);    

           
      if (keysym == XK_KP_Begin || keysym == XK_Begin
                    || keysym == 0) keysym = XK_Execute;             

      if (keysym >= XK_KP_0 && keysym <= XK_KP_9)
                    keysym = (keysym - XK_KP_0) + '0'; 

      if (keysym >= XK_KP_Home && keysym <= XK_KP_End)
                    keysym = (keysym - XK_KP_Home) + XK_Home;
		    		          
      
      return (int) keysym;
}
#else
int  kbdGet(void)
{
   int key;
   int buffer[10];
   int *buf;

   key = vga_getkey();
   if (key == KY_ERASE) key = KY_BACK;
   if (key != Esc) return key;

   // Key == Esc look for special key
   buf = buffer;
   while((*buf++=vga_getkey()) != 0);

   buf = buffer;
   if ((key=*buf++) != 0x5b) return key ? 0x200 | key : Esc;    // Meta key
   if ((key=*buf++) != 0x5b) return 0x100 | key;    // Arrow key
   return 0x300 | *buf;			   // Functional key

}
#endif



/* Init joystick   */
/* Returns
    0 - all right
    4 - error
*/

int joyInit(void)
{

#ifdef JOYSTICK_SUPPORTED
     if (JoyFd >= 0) close(JoyFd);
     JoyFd = open ("/dev/js0", O_RDONLY | O_NONBLOCK);
     JoyButtonLocked = 0;

     JoyVersion = 0x000800;	
     JoyNewDriver = 0;

#ifdef NEW_JOYSTICK
    if (UseNewDriver && JoyFd >=0 && ioctl(JoyFd, JSIOCGVERSION, &JoyVersion) >= 0)
    {  JoyNewDriver = (JoyVersion >= 0x010208);
       if (JoyNewDriver)
	 {// fcntl(JoyFd, F_SETFL, O_NONBLOCK);
	   ioctl(JoyFd, JSIOCGNAME(31), &JoyName); 
	   JoyName[31] = '\0';
       }
    }
#endif

#else
     JoyFd = -1;
#endif

     return JoyFd < 0 ? 4: 0;
}

void joyDeinit(void)
{
     if (JoyFd >= 0) close (JoyFd);
     JoyFd = -1;
}

int joyGetStatus(void)
{
#ifdef JOYSTICK_SUPPORTED
      int  mask;

/*     if (JoyFd < 0)
      {  joyInit();
         if (JoyFd < 0) return 4;
      }
*/

#ifdef NEW_JOYSTICK
     if (JoyNewDriver)
     {   struct js_event joyEvent;
        __u8 eType, eNumber;
        __s16 eValue;
        char new_status = 0;

       while(read(JoyFd, &joyEvent, sizeof(struct js_event)) > 0)
       {	
	   new_status = 1;
         eType = joyEvent.type; 
	   eNumber = joyEvent.number;

  	   eValue = joyEvent.value;

	   switch (eType & ~JS_EVENT_INIT)
	   { 	case JS_EVENT_BUTTON:
                   if ( eNumber < 4)
	             { 
		   /*      if (!(eType & JS_EVENT_INIT))
					JoyEnabled = 1;  */
			   mask = 1 << eNumber;
                     if (eValue) Buttons |= mask;
                          else   Buttons &= ~mask;
                   }
                   break;
				
	
	      case JS_EVENT_AXIS:
		/*	 if (JoyEnabled) */
	 	       { if (eNumber == 0) JoyX = eValue;
		         else
                     if (eNumber == 1) JoyY = eValue;
		       }        
	   }
       }
    
       if (!new_status) return 0;
    }
    else
#endif
    {   struct JS_DATA_TYPE js;

        if (read (JoyFd, &js, JS_RETURN) != JS_RETURN) return 4;

        JoyX = js.x;
        JoyY = js.y;
 
        Buttons = js.buttons;
     }

      mask = 2;
      while (mask)
      {  // If button pressed, ignore it if locked
         //           or set locked flag if unlocked
         if (Buttons & mask)
         {  if (JoyButtonLocked & mask) Buttons &= ~mask;
                         else JoyButtonLocked |= mask;
         }
         // If button not checked - clear locked mask
         else
                JoyButtonLocked &= ~mask;

         mask >>= 1;
      }


      return 0;
#else
     return 4;
#endif
}

int joySetParams(void)
{
#ifdef JOYSTICK_SUPPORTED
#ifdef NEW_JOYSTICK
	if (JoyNewDriver)
	{ 
  	    joyGetStatus();

          JoyX2 = JoyX + (long)(NEW_JOY_MAX-JoyX)*NEW_JOYSTICK_THRESHOLD_X/NEW_JOY_MAX;
          JoyX1 = JoyX - (long)(NEW_JOY_MAX+JoyX)*NEW_JOYSTICK_THRESHOLD_X/NEW_JOY_MAX;
	    JoyY2 = JoyY + (long)(NEW_JOY_MAX-JoyY)*NEW_JOYSTICK_THRESHOLD_Y/NEW_JOY_MAX;
	    JoyY1 = JoyY - (long)(NEW_JOY_MAX+JoyY)*NEW_JOYSTICK_THRESHOLD_Y/NEW_JOY_MAX;
	}
	else
#endif
	{	

  	  if (joyGetStatus()) return 4;

	  JoyX1 = JoyX >> 1;
	  JoyY1 = JoyY >> 1;

	  if (JoyX1 == 0 && JoyY1 == 0) return 4;

	  JoyX2 = JoyX1 * 3;
	  JoyY2 = JoyY1 * 3;
	}
	return 0;
#else
	return 4;
#endif
}


static int getJoy(void)
{
#ifdef JOYSTICK_SUPPORTED

   if (joyGetStatus() == 0)
   {
        if (Buttons)
        {  JoyEnabled = 1;
           return (JOY_START | Buttons);	
        }
         
      /*  No joystick button is pressed */
	if (JoyY > JoyY2)
	{ if (JoyX >= JoyX1 && JoyX <= JoyX2)
                     return JOY_DOWN;
	}
	else
	if (JoyY < JoyY1)
	{ if (JoyX >= JoyX1 && JoyX <= JoyX2)
                     return JOY_UP;
	}
	else
	if (JoyX > JoyX2) return JOY_RIGHT;
	else
	if (JoyX < JoyX1) return JOY_LEFT;
  }
#endif

  return 0;
}


 /*  Get keyboard or joystick command */
int getKeyJoy(void)
{

   int ret = getJoy();

   if ( !JoyEnabled || !ret) ret = kbdGet();

   return ret;
}


static int IndOfInt(int where[], int what)
{  int i, k;

   for (i=0; (k=where[i])!= '\0'; i++)
       if (k==what) return i;

  return -1;
}


int checkKeys(int *keys)
{  int ind;
 

#ifdef JOYSTICK_SUPPORTED
  if (joyGetStatus() == 0)
  {

    if (Buttons & 0x1)
    {  if ((ind = IndOfInt(keys, JOY_BTN1)) >= 0)
                         return ind;
    }

    if (Buttons & 0x2)
    {  if ((ind = IndOfInt(keys, JOY_BTN2)) >= 0)
                         return ind;
    }

  }
#endif

  while ((ind = kbdGet()) != 0)
  {   if (ind < 0x100) ind = toupper(ind);
      if ((ind = IndOfInt(keys, ind)) >= 0)
               return ind;
  }

  return -1;
}

int IsJoystickSupported(void)
{  return (JoyFd < 0) ? 0 : 1; }

